
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <unistd.h>
#include <errno.h>

#include <sys/types.h> 
#include <sys/stat.h> 
#include <fcntl.h>

int IICfh;

#define XIIC_RESET_REG   0x40
#define XIIC_RESET_DATA   0x0a

#define XIIC_REG_OFFSET  0x100
#define XIIC_CR_REG_OFFSET 0x00+XIIC_REG_OFFSET   /* Control register */
#define XIIC_SR_REG_OFFSET 0x04+XIIC_REG_OFFSET   /* Status register */
#define XIIC_DTR_REG_OFFSET 0x08+XIIC_REG_OFFSET  /* Data TX register */
#define XIIC_DRR_REG_OFFSET 0x0c+XIIC_REG_OFFSET  /* Data RX register */
#define XIIC_RFD_REG_OFFSET 0x20+XIIC_REG_OFFSET

#define XIIC_SR_BUS_BUSY_MASK 0x04
#define XIIC_SR_RX_FIFO_EMPTY_MASK 0x40

#define IIC_ADDR 0x50
#define EEPROM_ADDR 0xfa

#define MAXTRIES 1000

int main(int argc, char **argv) {
   int rc;
    unsigned int Status;
   unsigned int Data;
   unsigned int Address;
   int i;
   int tries;
   unsigned char macaddr[6] = {255,255,255,255,255,255};


  IICfh = open("/dev/iic0", O_RDWR);
    sleep(1);


    lseek(IICfh, XIIC_RESET_REG , SEEK_SET);     // soft reset register 
    Data = XIIC_RESET_DATA;      //  soft reset key
    rc = write(IICfh, &Data, 4);
    printf("reset iic bus: rc= 0x%x\n",rc);

   sleep(1);

    lseek(IICfh, XIIC_RFD_REG_OFFSET , SEEK_SET); //   set Rx_FIFO depth
    Data = 15;
    rc = write(IICfh, &Data, 4);

    lseek(IICfh, XIIC_CR_REG_OFFSET , SEEK_SET);    //  control register
    Data = 2;       //  reset Tx_FIFO
    rc = write(IICfh, &Data, 4);

    lseek(IICfh, XIIC_CR_REG_OFFSET , SEEK_SET);    //  control register
    Data = 1;       //  enable; remove Tx_FIFO reset
    rc = write(IICfh, &Data, 4);

    lseek(IICfh, XIIC_CR_REG_OFFSET , SEEK_SET);    //  control register
    rc = read(IICfh, &Status, 4);                 //  read it
    printf("read control register: rc= 0x%x; value= 0x%x\n", rc, Status);

    lseek(IICfh, XIIC_SR_REG_OFFSET , SEEK_SET);    //  status register
    rc = read(IICfh, &Status, 4);                 //  read it
    printf("read status register: rc= 0x%x; status= 0x%x\n", rc, Status);

  if (!(Status & XIIC_SR_BUS_BUSY_MASK)) {

    lseek(IICfh, XIIC_CR_REG_OFFSET , SEEK_SET);    //  control register
    Data = 9;       //  enable; transmit
    rc = write(IICfh, &Data, 4);

    lseek(IICfh, XIIC_DTR_REG_OFFSET , SEEK_SET);     /* Data TX register */
    Data = (IIC_ADDR << 1) + 0x100;      /*  set start bit; device address; write access */
    rc = write(IICfh, &Data, 4);

    lseek(IICfh, XIIC_DTR_REG_OFFSET , SEEK_SET);     /* Data TX register */
    Data = EEPROM_ADDR;      /*   EEPROM address */
    rc = write(IICfh, &Data, 4);

    lseek(IICfh, XIIC_SR_REG_OFFSET , SEEK_SET);    //  status register
    rc = read(IICfh, &Status, 4);                 //  read it
    printf("read status register: rc= 0x%x; status= 0x%x\n", rc, Status);

    sleep(1);

    lseek(IICfh, XIIC_SR_REG_OFFSET , SEEK_SET);    //  status register
    rc = read(IICfh, &Status, 4);                 //  read it
    printf("read status register: rc= 0x%x; status= 0x%x\n", rc, Status);

    lseek(IICfh, XIIC_CR_REG_OFFSET , SEEK_SET);    //  control register
    Data = 5;       //  receive + MSMS=1 + enable
    rc = write(IICfh, &Data, 4);

    lseek(IICfh, XIIC_DTR_REG_OFFSET , SEEK_SET);     // Data TX register 
    Data = (IIC_ADDR << 1) + 1 + 0x100;      //  set start bit for repeated start; device address; read access
    rc = write(IICfh, &Data, 4);

    lseek(IICfh, XIIC_DTR_REG_OFFSET , SEEK_SET);     // Data TX register
    Data = 0x200 + 6;      //  set stop bit; 6 bytes to be received
    rc = write(IICfh, &Data, 4);

    i = 0;
    for (tries=MAXTRIES; tries>0; tries--) {

        lseek(IICfh, XIIC_SR_REG_OFFSET , SEEK_SET);    //  status register
        rc = read(IICfh, &Status, 4);                 //  read it
        if (!(Status & XIIC_SR_RX_FIFO_EMPTY_MASK)) {

            lseek(IICfh, XIIC_DRR_REG_OFFSET , SEEK_SET);     // Data RX register 
            rc = read(IICfh, &Data, 4);
            macaddr[i] = (unsigned char) (Data & 255);
            i++;
        }
        if (i == 6) break;
    }
    if (tries == 0) {
      printf("Failed to obtain MAC address\n");

      lseek(IICfh, XIIC_SR_REG_OFFSET , SEEK_SET);    //  status register
      rc = read(IICfh, &Status, 4);                 //  read it
      printf("read status register: rc= 0x%x; status= 0x%x\n", rc, Status);
    } else {
       printf("MAC address = ");
       for (i=0; i<6; i++) {printf("0x%x ",macaddr[i]);}
       printf("\n");
    }

  } else {
    printf("read 0x%x; iic bus is busy: status= 0x%x\n", rc, Status);
  }

  sleep(1);
  close(IICfh);

    exit(0);

}
